/*
 * FreeModbus Libary: Linux Port
 * Copyright (C) 2006 Christian Walter <wolti@sil.at>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * File: $Id: portevent.c,v 1.1 2006/08/01 20:58:49 wolti Exp $
 */

/* ----------------------- Modbus includes ----------------------------------*/
#include "mb.h"
#include "mbport.h"
#include "mbctx.h"
#include "mbtcp.h"

/* ----------------------- Variables ----------------------------------------*/
static eMBEventType eQueuedEvent;
static BOOL     xEventInQueue;

/* ----------------------- Start implementation -----------------------------*/
BOOL
xMBPortEventInit( fmodbus_t* ctx )
{
    ctx->xEventInQueue = FALSE;
    return TRUE;
}

BOOL
xMBPortEventPost(fmodbus_t* ctx, eMBEventType eEvent )
{
    ctx->xEventInQueue = TRUE;
    ctx->eQueuedEvent = eEvent;
    return TRUE;
}

BOOL
xMBPortEventGet(fmodbus_t* ctx, eMBEventType * eEvent )
{
    BOOL            xEventHappened = FALSE;

    if( ctx->xEventInQueue )
    {
        *eEvent = ctx->eQueuedEvent;
        ctx->xEventInQueue = FALSE;
        xEventHappened = TRUE;
    }
    else
    {
        if( ctx->ucMBAddress == MB_TCP_PSEUDO_ADDRESS)
        {
            /* We can't do anything with errors from the pooling module. */
            xMBPortTCPPool( ctx );
        }
        else
        {
            /* Poll the serial device. The serial device timeouts if no
             * characters have been received within for t3.5 during an
             * active transmission or if nothing happens within a specified
             * amount of time. Both timeouts are configured from the timer
             * init functions.
             */
            ( void )xMBPortSerialPoll( ctx );

            /* Check if any of the timers have expired. */
            vMBPortTimerPoll( ctx );
        }
    }
    return xEventHappened;
}
